package gss.supportcase;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.transform.stream.StreamSource;

import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.impl.client.DefaultHttpClient;
import org.jboss.resteasy.client.ClientExecutor;
import org.jboss.resteasy.client.ClientRequest;
import org.jboss.resteasy.client.ClientRequestFactory;
import org.jboss.resteasy.client.ClientResponse;
import org.jboss.resteasy.client.core.executors.ApacheHttpClient4Executor;
import org.jboss.resteasy.spi.ResteasyProviderFactory;
import org.kie.api.runtime.process.ProcessInstance;
import org.kie.api.task.model.Content;
import org.kie.api.task.model.TaskSummary;
import org.kie.internal.task.api.ContentMarshallerContext;
import org.kie.remote.client.jaxb.JaxbContentResponse;
import org.kie.remote.client.jaxb.JaxbTaskSummaryListResponse;
import org.kie.services.client.serialization.jaxb.impl.process.JaxbProcessInstanceResponse;


public class Case01442738 {


    private static final String URL = "http://localhost:8080/business-central/rest";
    private static final String USER = "jboss";
    private static final String PASSWORD = "123$qweR";
    
    private static final String DEPLOYMENT_ID = "org.acme:myartifact:1.0-SNAPSHOT";
    

	public static void main(String[] args) throws Exception {

		// Start a new process "TestProcess" on server side 
		String processDefId = "myartifact.TestProcess";
		ProcessInstance process = startProcess(processDefId);
		System.out.println("Process started, ID:" + process.getId());
		
		
		
		
		// Get the Tasks from process instance
		List<TaskSummary> lstTaskSummary = listTaskbyPID(process.getId());
		for (TaskSummary task : lstTaskSummary) {
			System.out.println("Process: "+process.getId() + ": has task ID:" + task.getId());
			
			
			
			// get contnet from a task, using "/task/{taskId}/content" API
			Content taskContent_xml = getTaskContentXML(task.getId());
			Content taskContent_json = getTaskContentJSON(task.getId());
		}
		
	}

	
	
	
	
	
	
	
    
    // http://docs.jboss.org/jbpm/v6.2/userguide/jBPMRemoteAPI.html#remote.rest.runtime
	// 17.2.1.1. Process calls
	// [POST] /runtime/{deploymentId}/process/{processDefId}/start
    public static ProcessInstance startProcess(String processDefId) throws Exception {
    	
    	String startProcessURL = URL + "/runtime/" + DEPLOYMENT_ID + "/process/" + processDefId + "/start";
        System.out.println("---> Start Process via:" + startProcessURL);
        
        ClientRequest clientRequest = getClientRequestFactory().createRequest(startProcessURL);
        
        // Get response and extract returned `ProcessInstance` object
        ClientResponse<?> responseObj = (ClientResponse<?>) checkResponse(clientRequest.post());
        InputStream isResponse = responseObj.getEntity(InputStream.class);
        StreamSource source  = new StreamSource(isResponse);
        JAXBContext jaxbTaskContext = JAXBContext.newInstance(JaxbProcessInstanceResponse.class);
        
        // unmarshal to get the final ProcessInstance object
        ProcessInstance process = jaxbTaskContext.createUnmarshaller().unmarshal(source, JaxbProcessInstanceResponse.class).getValue();
    	return process; 
    }
    
    
    
    // http://docs.jboss.org/jbpm/v6.2/userguide/jBPMRemoteAPI.html#remote.rest.task
    // 17.2.3.2. Task query call
    // [GET] /task/query
    public static List<TaskSummary> listTaskbyPID(long processId) throws Exception {
    	
    	String startProcessURL = URL + "/task/query?processInstanceId="+processId;
        System.out.println("---> Query Task List by process ID via:" + startProcessURL);
        
        ClientRequest clientRequest = getClientRequestFactory().createRequest(startProcessURL);
        ClientResponse<?> responseObj = (ClientResponse<?>) checkResponse(clientRequest.get());
        InputStream isResponse = responseObj.getEntity(InputStream.class);
        
        
        StreamSource source  = new StreamSource(isResponse);
        JAXBContext jaxbTaskContext = JAXBContext.newInstance(JaxbTaskSummaryListResponse.class);
        
        List<TaskSummary> lstTaskSummary = jaxbTaskContext.createUnmarshaller().unmarshal(source, JaxbTaskSummaryListResponse.class).getValue().getResult();
    	return lstTaskSummary; 
    }
    
    
    
    
    

    // http://docs.jboss.org/jbpm/v6.2/userguide/jBPMRemoteAPI.html#remote.rest.task
    // 17.2.3.3. Other Task calls
    // [GET] /task/{taskId}/content
    public static Content getTaskContentXML(long taskId) throws Exception {
    	
    	String startProcessURL = URL + "/task/" + taskId + "/content";
        System.out.println("---> Query Tasks Content via:" + startProcessURL);

        Content taskContent = null;
        
        ClientRequest clientRequest = getClientRequestFactory().createRequest(startProcessURL);
        
        ClientResponse<?> responseObj = (ClientResponse<?>) checkResponse(clientRequest.get());
        InputStream isResponse = responseObj.getEntity(InputStream.class);
        

        
        
        
        StreamSource source  = new StreamSource(isResponse);
        JAXBContext jaxbTaskContext = JAXBContext.newInstance(JaxbContentResponse.class);
        Content content = jaxbTaskContext.createUnmarshaller().unmarshal(source, JaxbContentResponse.class).getValue().getResult();
        
        System.out.println("Get content object:"+content.toString());
        
        
        // TODO:
        // "content" field is serialized and it seems we need to de-serialize it ourself 
        // some explanation in this doc page: http://docs.jboss.org/jbpm/v6.2/userguide/jBPMTaskService.html#jBPMTaskServiceAPI
        // reading this doc, and the src of Content class, as I understand it saves the custom object into content field as "base64Binary" format binary data, and if you use .getContent() that's what you get. 
        
        
        
        return taskContent;
        
    }
    
    
    
    // http://docs.jboss.org/jbpm/v6.2/userguide/jBPMRemoteAPI.html#remote.rest.task
    // 17.2.3.3. Other Task calls
    // [GET] /task/{taskId}/content
    public static Content getTaskContentJSON(long taskId) throws Exception {
    	
    	String startProcessURL = URL + "/task/" + taskId + "/content";
        System.out.println("---> Query Tasks Content via:" + startProcessURL);

        Content taskContent = null;
        
        ClientRequest clientRequest = getClientRequestFactory().createRequest(startProcessURL);
        clientRequest.accept("application/json");
        
        ClientResponse<?> responseObj = (ClientResponse<?>) checkResponse(clientRequest.get());
        InputStream isResponse = responseObj.getEntity(InputStream.class);
        
        
        
        BufferedReader reader = new BufferedReader(new InputStreamReader(isResponse));
        String line;
        System.out.println("***** Printing REST Return *****");   
        while ((line = reader.readLine()) != null) {
        	System.out.println(line);  
        }
        reader.close();
        

        
        
        return taskContent;
        
    }
    
    
    
    
    
    
    
    
    
	public static ClientResponse<?> checkResponse(ClientResponse<?> responseObj) throws Exception {
		responseObj.resetStream();
		int status = responseObj.getStatus();
		if (status != 200) {
			throw new Exception("Response with exception:\n" + responseObj.getEntity(String.class));
		}
		return responseObj;
	}
	
	private static ClientRequestFactory getClientRequestFactory() {
		DefaultHttpClient httpClient = new DefaultHttpClient();
		httpClient.getCredentialsProvider().setCredentials(new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM), new UsernamePasswordCredentials(USER, PASSWORD));
		ClientExecutor clientExecutor = new ApacheHttpClient4Executor(httpClient);
		
		return new ClientRequestFactory(clientExecutor, ResteasyProviderFactory.getInstance());
	}

}